/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "NodeIterator.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomNode.h"
#include "gdom/GdomNodeIterator.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomNode.h"
#include "gdom/PrivateGdomNodeIterator.h"
#include "gdom/PrivateIntGdomNode.h"
#include "gdom/PrivateIntGdomNodeIterator.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::NodeIterator* obj)
{
    return WebCore::cacheDOMObject<WebCore::NodeIterator, GDOMNodeIterator>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMNodeIterator::GDOMNodeIterator(PassRefPtr<WebCore::NodeIterator> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMNodeIterator::~GDOMNodeIterator()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}


/* XXX TODO: custom function gdom_node_iterator_next_node */


/* XXX TODO: custom function gdom_node_iterator_previous_node */

void
gdom_node_iterator_detach (GdomNodeIterator *thiz)
{
    g_return_if_fail (thiz);
    WebCore::NodeIterator * item = (gdom_node_iterator_get_private(thiz));
    g_return_if_fail (item);
    item->detach();

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "NodeIterator.h"
#include <gdom/GdomNodeIterator.h>
#include <gdom/PrivateGdomNodeIterator.h>
#include <gdom/PrivateIntGdomNodeIterator.h>
#include <gdom/GdomNodeIteratorprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomNodeIterator, gdom_node_iterator, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomNodeIteratorPrivate {
    class WebCore::NodeIterator *item;
};

#define GDOM_NODE_ITERATOR_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_NODE_ITERATOR, GdomNodeIteratorPrivate))
*/
G_END_DECLS
WebCore::NodeIterator *
gdom_node_iterator_get_private(GdomNodeIterator *request)
{
    GDOMNodeIterator *res;
    res = static_cast<GDOMNodeIterator*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::NodeIterator*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_ROOT,
    PROP_WHAT_TO_SHOW,
    PROP_FILTER,
    PROP_EXPAND_ENTITY_REFERENCES,
    PROP_REFERENCE_NODE,
    PROP_POINTER_BEFORE_REFERENCE_NODE,
};


static void gdom_node_iterator_finalize(GObject* object)
{
    /*GdomNodeIterator* request = GDOM_NODE_ITERATOR(object);*/
    /*GdomNodeIteratorPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_node_iterator_parent_class)->finalize(object);
}

static void gdom_node_iterator__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomNodeIterator *self = GDOM_NODE_ITERATOR(object);
    WebCore::NodeIterator * gself = (gdom_node_iterator_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_node_iterator__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomNodeIterator *self = GDOM_NODE_ITERATOR(object);
    WebCore::NodeIterator * gself = (gdom_node_iterator_get_private(self));
    switch (prop_id) {
    case PROP_ROOT:
    {
         PassRefPtr<WebCore::Node> ptr = gself->root();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_WHAT_TO_SHOW:
    {
         g_value_set_ulong (value, gself->whatToShow());
         break;
    }
    case PROP_FILTER:
    {
         PassRefPtr<WebCore::NodeFilter> ptr = gself->filter();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_EXPAND_ENTITY_REFERENCES:
    {
         g_value_set_boolean (value, gself->expandEntityReferences());
         break;
    }
    case PROP_REFERENCE_NODE:
    {
         PassRefPtr<WebCore::Node> ptr = gself->referenceNode();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_POINTER_BEFORE_REFERENCE_NODE:
    {
         g_value_set_boolean (value, gself->pointerBeforeReferenceNode());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_node_iterator_class_init(GdomNodeIteratorClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_node_iterator_finalize;
    g_klass->set_property = gdom_node_iterator__set_property_fn;
    g_klass->get_property = gdom_node_iterator__get_property_fn;

    g_object_class_install_property (g_klass, PROP_ROOT,
            g_param_spec_object ("root", /* name */
                "node_iterator_root", /* short description */
                "read-only  GdomNode * NodeIterator.root", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_WHAT_TO_SHOW,
            g_param_spec_ulong ("what_to_show", /* name */
                "node_iterator_what_to_show", /* short description */
                "read-only  gulong NodeIterator.what_to_show", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_FILTER,
            g_param_spec_object ("filter", /* name */
                "node_iterator_filter", /* short description */
                "read-only  GdomNodeFilter * NodeIterator.filter", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE_FILTER, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_EXPAND_ENTITY_REFERENCES,
            g_param_spec_boolean ("expand_entity_references", /* name */
                "node_iterator_expand_entity_references", /* short description */
                "read-only  gboolean NodeIterator.expand_entity_references", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_REFERENCE_NODE,
            g_param_spec_object ("reference_node", /* name */
                "node_iterator_reference_node", /* short description */
                "read-only  GdomNode * NodeIterator.reference_node", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_POINTER_BEFORE_REFERENCE_NODE,
            g_param_spec_boolean ("pointer_before_reference_node", /* name */
                "node_iterator_pointer_before_reference_node", /* short description */
                "read-only  gboolean NodeIterator.pointer_before_reference_node", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomNodeIteratorPrivate));*/
}

static void gdom_node_iterator_init(GdomNodeIterator* request)
{
    /*GdomNodeIteratorPrivate* priv = GDOM_NODE_ITERATOR_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

